'----------------------------------------------------------------------------------------
'    Name:  TLTime_i.INC
'    Type:  TIGER-BASIC(tm) Source Code
' Release:  V2.04
' Package:  Tiger BASIC Library
'
' Purpose:  Time functions
'
'  (C) - Copyright Wilke Technology, P.O.Box 101812, D-52018 Aachen, Germany
'----------------------------------------------------------------------------------------
'
' Thank you for using BASIC-Tiger(tm) in your products. If you have questions, ideas
' or special needs, please contact your next distributor or the Tiger support team
' and visit our web site:
'
'    Wilke Technology GmbH
'    The Tiger Support Team
'    P.O.Box 101812,     D-52018 Aachen, Germany
'    Heider-Hof-Weg 23D, D-52080 Aachen, Germany
'
'    email: support@wilke-technology.com  (English)
'    email: support@wilke.de              (German)
'    Phone: +49 (2405) 408 55 0    Mo to Fr, 7:00 to 16:00 (GMT)
'    Fax:   +49 (2405) 408 55 444
'
' New information, new drivers and free downloads see:
'
'    www.wilke-technology.com     (English)
'    www.wilke.de                 (German)
'
' Sincerely,
'
'    Your Tiger Support Team
'
'----------------------------------------------------------------------------------------
'
' List of currently included functions:
'
'   vTLTimeCVT_init   initializes all global variables for time conversion with starting values
'
'   lTLDos2Tig		    converts date and time into seconds since epoch
'
'   lTLTig2DOS        converts time from Tiger format into single values for time and date
'
'   lTLConv_Duration  calculates time span in days, hours, minutes, seconds and milliseconds
'                     from a value containing milliseconds
'
'   bTLIsDst          calculates if given date and hour are inside of european daylight saving time
'
'   bTLCalcDayOfWeek  calculates the day of the week for a given date
'
'   bTLIsLeapYear     checks if given year is a leap year
'
'----------------------------------------------------------------------------------------
#ifndef __TLTime_i_INC
#define __TLTime_i_INC

#include TigerBASICLibraryDefs.INC
#include define_a.inc
#include TLTime_d.INC


byte  bgTLDaylightSavingTime                  ' flag for daylight saving time
array sgTLYMonthsShort$(13,3) of strings(4)   ' output names for month (short name)
array sgTLYMonths$(13,3) of strings(9)        ' output names for month (long name)
array sgTLWDay$(7,3) of strings(10)           ' day of week (long name)
array sgTLWDayShort$(7,3) of strings(3)       ' day of week (short name)
array lgTLDaysInMonth(2, 12) of long          ' number of days in month
word  wgEpoch
byte  bgDaylightSavingTimeCorrection


'----------------------------------------------------------------------------------------
' Subroutine:  Converts date and time into seconds since epoch
'----------------------------------------------------------------------------------------
'
'   input:
'   bpSec          <byte> = seconds
'   bpMin          <byte> = minutes
'   bpHour         <byte> = hours
'   bpDay          <byte> = day
'   bpMon          <byte> = month
'   wpYear         <word> = year
'
'   output:  
'   lpvTimestamp   <long> = number of seconds since epoch
'   wpvError       <word> = error code
'
'----------------------------------------------------------------------------------------
sub lTLDos2Tig (byte bpSec, bpMin, bpHour, bpDay, bpMon; word wpYear; var long lpvTimestamp; var word wpvError)
  byte blIsLeapYear
  byte blTLMDaysIdx
  
  call bTLIsLeapYear(wpYear, blIsLeapYear)
  
  if blIsLeapYear = TRUE then
    blTLMDaysIdx = 1
  else
    blTLMDaysIdx = 0
  endif
  
  wpvError = TL_ERR_OK
  
  ' check if all parameters are valid
  if bpSec >= 60 then                      ' Seconds out of range?
    wpvError = TL_ERR_TIME_INVALID_SECOND
    return
  endif
  if bpMin >= 60 then                      ' Minutes out of range?
    wpvError = TL_ERR_TIME_INVALID_MINUTE
    return
  endif
  if bpHour >= 24 then                     ' Hours out of range?
    wpvError = TL_ERR_TIME_INVALID_HOUR
    return
  endif
  if wpYear > 3000 then                    ' Year out of range?
    wpvError = TL_ERR_TIME_INVALID_YEAR
    return
  endif
  if bpMon < 1 OR bpMon > 12 then          ' Month out of range?
    wpvError = TL_ERR_TIME_INVALID_MONTH
    return
  endif
  if bpDay < 1 OR bpDay > lgTLDaysInMonth(blTLMDaysIdx, bpMon-1) then      ' Days out of range?
    wpvError = TL_ERR_TIME_INVALID_DAY
    return
  endif
  
  '' calculate the seconds since or to epoch
  '' formula for dates before epoch will subtract the full years first, and then add the days passed in that year.  
  long llSign 
  word wlI
  word wlFrom, wlTo
  
  if wpYear >= wgEpoch  then
    llSign = 1
    wlFrom = wgEpoch
    wlTo = wpYear
  else
    llSIgn = -1
    wlFrom = wpYear
    wlTo = wgEpoch
  endif
  
  long llNleapdays   '' The number of leap days
  long llMonthsNDays '' The number of days passed in the current year
  
  llNleapdays = 0
  llMonthsNDays = 0
  
  '' count the leap days
  for wlI = wlFrom to wlTo - 1
    call bTLIsLeapYear(wlI, blIsLeapYear)
    if blIsLeapYear = TRUE then
      llNleapdays = llNleapdays + 1
    endif
  next
  
  '' calculate the number of days passed in the current year
  for wlI = 1 to bpMon - 1
    llMonthsNDays = llMonthsNDays + lgTLDaysInMonth(blTLMDaysIdx, wlI-1)
  next
  
  lpvTimestamp = ( ( wpYear - wgEpoch ) * 365 + llSign * llNleapdays + llMonthsNDays + bpDay - 1 ) * 86400 + bpHour * 3600 + bpMin * 60 + bpSec
  
  '' daylight saving time
  if bgDaylightSavingTimeCorrection = TRUE then
    byte blDayOfWeek
    byte blIsDst
    
    call bTLCalcDayOfWeek( bpDay, bpMon, wpYear, blDayOfWeek ) 
    call bTLIsDst( bpDay, bpMon, blDayOfWeek, bpHour, blIsDst)
    if blIsDst = 1 then                   ' if it is daylight saving time
      lpvTimestamp = lpvTimestamp - 3600  ' we substract one hour resp. 3600 seconds for the RTC
    endif
    bgTLDaylightSavingTime = blIsDst      ' now set global variable for DST  
  endif
end

'----------------------------------------------------------------------------------------
' Subroutine:  Check if given year is a leap year
'----------------------------------------------------------------------------------------
'
'   input:
'   wpYear         <word> = year
'
'   output:  
'   bpvIsLeapYear  <byte> = TRUE if given year is a leap year
'
'----------------------------------------------------------------------------------------
sub bTLIsLeapYear(word wpYear; var byte bpvIsLeapYear)
  if mod(wpYear, 400) = 0 OR ( mod(wpYear, 4) = 0 AND mod(wpYear, 100) <> 0 ) then
    bpvIsLeapYear = TRUE
  else
    bpvIsLeapYear = FALSE
  endif
end

'----------------------------------------------------------------------------------------
' Subroutine:  Calculates the day of the week for a given date
'----------------------------------------------------------------------------------------
'
'   input:
'   bpDay          <byte> = day
'   bpMonth        <byte> = month
'   wpYear         <word> = year
'
'   output:
'   bpvDayOfWeek   <byte> = day of week (0...6, 0 = sunday)
'
'----------------------------------------------------------------------------------------
sub bTLCalcDayOfWeek( byte bpDay, bpMonth; word wpYear; var byte bpvDayOfWeek ) 
  long q
  long m
  long k
  long j
  long h
  
  byte blMonth
  word wlYear
  blMonth = bpMonth
  wlYear = wpYear

  if blMonth = 1 then
    blMonth = 13
    wlYear = wlYear - 1
  endif
  if blMonth = 2 then
    blMonth = 14
    wlYear = wlYear - 1
  endif

  q = bpDay
  m = blMonth
  k = mod(wlYear, 100)
  j = wlYear / 100 
  h = q + 13*(m+1)/5 + k + k/4 + j/4 + 5*j
  h = mod(h, 7)
  
  switch h
    case 0 : bpvDayOfWeek = 6   ' Saturday
    case 1 : bpvDayOfWeek = 0   ' Sunday
    case 2 : bpvDayOfWeek = 1   ' Monday
    case 3 : bpvDayOfWeek = 2   ' Tuesday
    case 4 : bpvDayOfWeek = 3   ' Wednesday
    case 5 : bpvDayOfWeek = 4   ' Thursday
    case 6 : bpvDayOfWeek = 5   ' Friday
  endswitch
end

'----------------------------------------------------------------------------------------
' Subroutine:  Calculates if given date and hour are inside of european daylight saving time
'----------------------------------------------------------------------------------------
'
'   input:
'   bpDay          <byte> = day
'   bpMonth        <byte> = month
'   bpDayOfWeek    <byte> = day of week (0...6, 0 = sunday)
'   bpHour         <byte> = hours
'
'   output:
'   bpvIsDst       <byte> = TRUE if given moment is inside of european daylight saving time
'
'----------------------------------------------------------------------------------------
sub bTLIsDst(byte bpDay, bpMonth, bpDayOfWeek, bpHour; var byte bpvIsDst)
  long llDstDay

  if bpMonth < 3 OR bpMonth > 10 then    ' this is DST for sure, so we are ready
    bpvIsDst = FALSE                     ' set flag for DST
    return
  endif
  
  if bpMonth > 3 AND bpMonth < 10 then   ' this is DST for sure, so we are ready
    bpvIsDst = TRUE                      ' set flag for DST
    return
  endif
  
  long prevSunday
  prevSunday = bpDay - bpDayOfWeek
  
  if bpMonth = 3 then
    if prevSunday >= 25 then
      if bpDayOfWeek = 0 AND bpHour < 2 then
        bpvIsDst = FALSE
      else
        bpvIsDst = TRUE
      endif
    else
      bpvIsDst = FALSE
    endif
    return
  endif
  
  if bpMonth = 10 then
    if prevSunday < 25 then
      bpvIsDst = TRUE
    else
      if bpDayOfWeek = 0 AND bpHour < 2 then
        bpvIsDst = TRUE
      else
        bpvIsDst = FALSE
      endif
    endif
    return
  endif
end

'----------------------------------------------------------------------------------------
' Subroutine:  Converts time in Tiger format (seconds since epoch)
'    into single values for time and date
'----------------------------------------------------------------------------------------
'
'   input:
'   Seconds     <long> = number of seconds since epoch
'
'   output:
'   lpvSec      <byte> = seconds (0...60)
'   lpvMin      <byte> = minutes (0...60)
'   lpvHour     <byte> = hours   (0...23)
'   lpvDay      <byte> = day     (1...31)
'   lpvMon      <byte> = month   (1...12)
'   lpvYear     <byte> = year    (1980...2048)
'   lpvDoW      <byte> = weekday (0..6)
'   wpvError    <word> = error code
'
'----------------------------------------------------------------------------------------
SPEEDSUB lTLTig2DOS(long lpSeconds;var byte bpvSec,bpvMin,bpvHour,bpvDay,bpvMon;var word wpvYear; var byte bpvDoW; var word wpvError)
  long I ' loop variable
  long llRemainingHours, llHoursPerYear, llDays, llMonths
  byte blIsLeapYear, blTempMonth, blDst
  word wlYear, wlTempYear
  
  wpvError = TL_ERR_OK
  '
  ' !!!!!!!! check if parameter is valid !!!!!!!!
  '
  if lpSeconds < 0 then ' Tiger Seconds out of range?
    bpvSec  = -1
    bpvMin  = -1
    bpvHour = -1
    bpvDay  = -1
    bpvMon  = -1
    wpvYear = -1
    bpvDoW  = -1
    wpvError = TL_ERR_TIME_INVALID_TIGER_SEC
    return
  endif
  '
  ' !!!!!!!! actual conversion !!!!!!!!
  '
  bpvSec = mod(lpSeconds, 60)
  bpvMin = mod(lpSeconds / 60, 60)
  
  llRemainingHours = lpSeconds / 3600
  wlYear = wgEpoch
  for I = 0 to 1 step 0
    call bTLIsLeapYear(wlYear, blIsLeapYear)
    if blIsLeapYear = TRUE then
      llHoursPerYear = 8784
    else
      llHoursPerYear = 8760
    endif
    if llRemainingHours >= llHoursPerYear then
      wlYear = wlYear + 1
      llRemainingHours = llRemainingHours - llHoursPerYear
    else
      I = 2
    endif
  next
  
  wpvYear = wlYear
  bpvHour = mod(llRemainingHours, 24)
  llDays = (llRemainingHours / 24) + 1
  
  llMonths = 1
  call bTLIsLeapYear(wlYear, blIsLeapYear)
  for I = 0 to 11 step 1
    if lgTLDaysInMonth(blIsLeapYear, I) >= llDays then
        I = 12
    else
      llDays = llDays - lgTLDaysInMonth(blIsLeapYear, I)
      llMonths = llMonths +1
    endif
  next
  
  bpvMon = llMonths
  bpvDay = llDays
  
  '
  ' !!!!!!!! from here added for day of week !!!!!!!!
  '
  call bTLCalcDayOfWeek( bpvDay, bpvMon, wlYear, bpvDoW)
  
  '
  ' !!!!!!!! from here added for daylight saving time check !!!!!!!!
  '
  blDst = FALSE
  if bgDaylightSavingTimeCorrection = TRUE then
    call bTLIsDst(bpvDay, bpvMon, bpvDoW, bpvHour, blDst)
    if blDst = TRUE then        ' if it is daylight saving time
      bpvHour = bpvHour + 1  ' add one hour
      if bpvHour > 23 then   ' if the day changes because of that
        bpvHour = 0          ' set hour to 0
        bpvDay = bpvDay + 1  ' increase day
        if bpvDay > lgTLDaysInMonth(blIsLeapYear, bpvMon-1) then    ' if the month changes because of that
          bpvDay = 1          ' set day to 1
          bpvMon = bpvMon + 1 ' increase month
        endif
      endif
    endif
  endif
  bgTLDaylightSavingTime = blDst      ' now set global variable for DST
end

'----------------------------------------------------------------------------------------
' Subroutine: Initializes all global variables with starting values
'----------------------------------------------------------------------------------------
'
'   input:
'   wpEpoch           <word> = sets epoch base year (eg. 1980)
'   bpDSTCorrection   <byte> = 1: activates automatic Daylight saving time correction
'
' for non asscii characters see ISO/IEC 8859-1!
'----------------------------------------------------------------------------------------
sub vTLTimeCVT_init (word wpEpoch; byte bpDSTCorrection)
  lgTLDaysInMonth(0, 0) = 31
  lgTLDaysInMonth(0, 1) = 28
  lgTLDaysInMonth(0, 2) = 31
  lgTLDaysInMonth(0, 3) = 30
  lgTLDaysInMonth(0, 4) = 31
  lgTLDaysInMonth(0, 5) = 30
  lgTLDaysInMonth(0, 6) = 31
  lgTLDaysInMonth(0, 7) = 31
  lgTLDaysInMonth(0, 8) = 30
  lgTLDaysInMonth(0, 9) = 31
  lgTLDaysInMonth(0, 10) = 30
  lgTLDaysInMonth(0, 11) = 31
  
  '' Leap year
  lgTLDaysInMonth(1, 0) = 31
  lgTLDaysInMonth(1, 1) = 29
  lgTLDaysInMonth(1, 2) = 31
  lgTLDaysInMonth(1, 3) = 30
  lgTLDaysInMonth(1, 4) = 31
  lgTLDaysInMonth(1, 5) = 30
  lgTLDaysInMonth(1, 6) = 31
  lgTLDaysInMonth(1, 7) = 31
  lgTLDaysInMonth(1, 8) = 30
  lgTLDaysInMonth(1, 9) = 31
  lgTLDaysInMonth(1, 10) = 30
  lgTLDaysInMonth(1, 11) = 31
  
  
  sgTLYMonthsShort$(1,0)  = "Jan"      ' --,
  sgTLYMonthsShort$(2,0)  = "Feb"      '   |
  sgTLYMonthsShort$(3,0)  = "M<E4h>r"  '   | 
  sgTLYMonthsShort$(4,0)  = "Apr"      '   |
  sgTLYMonthsShort$(5,0)  = "Mai"      '   |
  sgTLYMonthsShort$(6,0)  = "Jun"      '   |
  sgTLYMonthsShort$(7,0)  = "Jul"      '   |-- German
  sgTLYMonthsShort$(8,0)  = "Aug"      '   |
  sgTLYMonthsShort$(9,0)  = "Sep"      '   |
  sgTLYMonthsShort$(10,0) = "Okt"      '   |
  sgTLYMonthsShort$(11,0) = "Nov"      '   |
  sgTLYMonthsShort$(12,0) = "Dez"      ' --
  sgTLYMonthsShort$(1,1)  = "Jan"      ' --,
  sgTLYMonthsShort$(2,1)  = "Feb"      '   |
  sgTLYMonthsShort$(3,1)  = "Mar"      '   |
  sgTLYMonthsShort$(4,1)  = "Apr"      '   |
  sgTLYMonthsShort$(5,1)  = "May"      '   |
  sgTLYMonthsShort$(6,1)  = "Jun"      '   |
  sgTLYMonthsShort$(7,1)  = "Jul"      '   |-- English
  sgTLYMonthsShort$(8,1)  = "Aug"      '   |
  sgTLYMonthsShort$(9,1)  = "Sep"      '   |
  sgTLYMonthsShort$(10,1) = "Oct"      '   |
  sgTLYMonthsShort$(11,1) = "Nov"      '   |
  sgTLYMonthsShort$(12,1) = "Dec"      ' --
  sgTLYMonthsShort$(1,2)  = "janv"     ' --,
  sgTLYMonthsShort$(2,2)  = "f<E8h>v"  '   | 
  sgTLYMonthsShort$(3,2)  = "mars"     '   |
  sgTLYMonthsShort$(4,2)  = "avr"      '   |
  sgTLYMonthsShort$(5,2)  = "mai"      '   |
  sgTLYMonthsShort$(6,2)  = "juin"     '   |
  sgTLYMonthsShort$(7,2)  = "juil"     '   |-- French
  sgTLYMonthsShort$(8,2)  = "ao<FBh>t" '   | 
  sgTLYMonthsShort$(9,2)  = "sept"     '   |
  sgTLYMonthsShort$(10,2) = "oct"      '   |
  sgTLYMonthsShort$(11,2) = "nov"      '   |
  sgTLYMonthsShort$(12,2) = "d<E9h>c"  ' -- 
  sgTLYMonths$(1,0)  = "Januar"       ' --,
  sgTLYMonths$(2,0)  = "Februar"      '   |
  sgTLYMonths$(3,0)  = "M<E4h>rz"     '   | 
  sgTLYMonths$(4,0)  = "April"        '   |
  sgTLYMonths$(5,0)  = "Mai"          '   |
  sgTLYMonths$(6,0)  = "Juni"         '   |
  sgTLYMonths$(7,0)  = "Juli"         '   |-- German
  sgTLYMonths$(8,0)  = "August"       '   |
  sgTLYMonths$(9,0)  = "September"    '   |
  sgTLYMonths$(10,0) = "Oktober"      '   |
  sgTLYMonths$(11,0) = "November"     '   |
  sgTLYMonths$(12,0) = "Dezember"     ' --
  sgTLYMonths$(1,1)  = "January"      ' --,
  sgTLYMonths$(2,1)  = "February"     '   |
  sgTLYMonths$(3,1)  = "March"        '   |
  sgTLYMonths$(4,1)  = "April"        '   |
  sgTLYMonths$(5,1)  = "May"          '   |
  sgTLYMonths$(6,1)  = "June"         '   |
  sgTLYMonths$(7,1)  = "July"         '   |-- English
  sgTLYMonths$(8,1)  = "August"       '   |
  sgTLYMonths$(9,1)  = "September"    '   |
  sgTLYMonths$(10,1) = "October"      '   |
  sgTLYMonths$(11,1) = "November"     '   |
  sgTLYMonths$(12,1) = "December"     ' --
  sgTLYMonths$(1,2)  = "janvier"      ' --,
  sgTLYMonths$(2,2)  = "f<E8h>vrier"  '   | 
  sgTLYMonths$(3,2)  = "mars"         '   |
  sgTLYMonths$(4,2)  = "avril"        '   |
  sgTLYMonths$(5,2)  = "mai"          '   |
  sgTLYMonths$(6,2)  = "juin"         '   |
  sgTLYMonths$(7,2)  = "juillet"      '   |-- French
  sgTLYMonths$(8,2)  = "ao<FBh>t"     '   | 
  sgTLYMonths$(9,2)  = "septembre"    '   |
  sgTLYMonths$(10,2) = "octobre"      '   |
  sgTLYMonths$(11,2) = "novembre"     '   |
  sgTLYMonths$(12,2) = "d<E8h>cembre" ' -- 
  sgTLWDayShort$(0,0) = "So"  ' --,
  sgTLWDayShort$(1,0) = "Mo"  '   |
  sgTLWDayShort$(2,0) = "Di"  '   |
  sgTLWDayShort$(3,0) = "Mi"  '   |-- German
  sgTLWDayShort$(4,0) = "Do"  '   |
  sgTLWDayShort$(5,0) = "Fr"  '   |
  sgTLWDayShort$(6,0) = "Sa"  ' --
  sgTLWDayShort$(0,1) = "Sun" ' --,
  sgTLWDayShort$(1,1) = "Mon" '   |
  sgTLWDayShort$(2,1) = "Tue" '   |
  sgTLWDayShort$(3,1) = "Wed" '   |-- English
  sgTLWDayShort$(4,1) = "Thu" '   |
  sgTLWDayShort$(5,1) = "Fri" '   |
  sgTLWDayShort$(6,1) = "Sat" ' --
  sgTLWDayShort$(0,2) = "di"  ' --,
  sgTLWDayShort$(1,2) = "lu"  '   |
  sgTLWDayShort$(2,2) = "ma"  '   |
  sgTLWDayShort$(3,2) = "me"  '   |-- French
  sgTLWDayShort$(4,2) = "je"  '   |
  sgTLWDayShort$(5,2) = "ve"  '   |
  sgTLWDayShort$(6,2) = "sa"  ' --
  sgTLWDay$(0,0) = "Sonntag"    ' --,
  sgTLWDay$(1,0) = "Montag"     '   |
  sgTLWDay$(2,0) = "Dienstag"   '   |
  sgTLWDay$(3,0) = "Mittwoch"   '   |-- German
  sgTLWDay$(4,0) = "Donnerstag" '   |
  sgTLWDay$(5,0) = "Freitag"    '   |
  sgTLWDay$(6,0) = "Samstag"    ' --
  sgTLWDay$(0,1) = "Sunday"     ' --,
  sgTLWDay$(1,1) = "Monday"     '   |
  sgTLWDay$(2,1) = "Tuesday"    '   |
  sgTLWDay$(3,1) = "Wednesday"  '   |-- English
  sgTLWDay$(4,1) = "Thursday"   '   |
  sgTLWDay$(5,1) = "Friday"     '   |
  sgTLWDay$(6,1) = "Saturday"   ' --
  sgTLWDay$(0,2) = "dimanche"   ' --,
  sgTLWDay$(1,2) = "lundi"      '   |
  sgTLWDay$(2,2) = "mardi"      '   |
  sgTLWDay$(3,2) = "mercredi"   '   |-- French
  sgTLWDay$(4,2) = "jeudi"      '   |
  sgTLWDay$(5,2) = "vendredi"   '   |
  sgTLWDay$(6,2) = "samedi"     ' --
  
  wgEpoch = wpEpoch
  bgDaylightSavingTimeCorrection = bpDSTCorrection
end

'----------------------------------------------------------------------------------------
' Subroutine:  calculates time span in days, hours, minutes, seconds and milliseconds
'    from a value containing milliseconds
'----------------------------------------------------------------------------------------
'
'   input:  lpDuration  <long> = duration in milliseconds
'
'   output:
'   bpDays     <long> = duration in days
'   bpHours    <long> = remaining duration in hours
'   bpMinutes  <long> = remaining duration in minutes
'   bpSeconds  <long> = remaining duration in seconds
'   wpvMilli   <long> = remianing duration in milliseconds
'   wpvError   <word> = error code
'
'----------------------------------------------------------------------------------------
sub lTLConv_Duration(long lpDuration; var long lpvMilli, lpvSeconds, lpvMinutes,&
  lpvHours, lpvDays; var word wpvError)
  
  long llmillisecs
  
  wpvError = TL_ERR_OK
  if lpDuration < 0 then
    lpvMilli = -1
    lpvSeconds = -1
    lpvMinutes = -1
    lpvHours = -1
    lpvDays = -1
    wpvError = TL_ERR_TIME_INVALID_DURATION
  else
    llmillisecs = lpduration
    lpvDays = (llmillisecs / 86400000)
    llmillisecs = llmillisecs - (lpvDays * 86400000)
    lpvHours = (llmillisecs / 3600000)
    llmillisecs = llmillisecs - (lpvHours * 3600000)
    lpvMinutes = (llmillisecs / 60000)
    llmillisecs = llmillisecs - (lpvMinutes * 60000)
    lpvSeconds = (llmillisecs / 1000)
    llmillisecs = llmillisecs - (lpvSeconds * 1000)
    lpvMilli = llmillisecs
  endif
end

#endif ' TLTime_i_INC
